searchNodes=[{"ref":"RTypes.html","title":"RTypes","type":"module","doc":"RTypes is an Elixir library which helps automatically create a validation function for a given user type. The function can be used to check the shape of the data after de-serialisation or in unit-tests. Let&#39;s suppose we have a type @type t :: 0..255 and we have a value x. To ensure that our value corresponds to the type t we can use the function def t?(x) when is_integer(x) and x &gt;= 0 and x &lt;= 255, do: true def t?(_), do: false Now, if we have a compound type @type list_of_ts :: [t] and a value xs, we can use is_list/1 guard on xs and then ensure that all elements of the list conform to t. And if we have a more complex structure @type state(a, b) :: %{key1: {a, b}, key2: list_of_ts()} and a value s, we can check that s is a map which has keys key1 and key2, apply the logic above for the value of key2 and for any concrete types a and b we can check that he value of key1 is a tuple of length 2 and its elements conform to a and b respectively. So we just recursively apply those checks. Usage The library defines make_validator/1 and make_predicate/1 macros, and make_validator/3 and make_predicate/3 functions which can be used at run time. The difference between the two is that a validator returns :ok or {:error, reason} where reason explains what went wrong, while a predicate returns only true or false and is somewhat faster. iex&gt; require RTypes iex&gt; port_number? = RTypes.make_predicate(:inet.port_number()) iex&gt; port_number?.(8080) true iex&gt; port_number?.(80000) false iex&gt; validate_is_kwlist = RTypes.make_validator(Keyword, :t, [{:type, 0, :pos_integer, []}]) iex&gt; validate_is_kwlist.(key1: 4, key2: 5) :ok iex&gt; match?({:error, _reason}, validate_is_kwlist.([1, 2, 3])) true"},{"ref":"RTypes.html#derive/1","title":"RTypes.derive/1","type":"macro","doc":""},{"ref":"RTypes.html#derive/3","title":"RTypes.derive/3","type":"function","doc":""},{"ref":"RTypes.html#derive!/1","title":"RTypes.derive!/1","type":"macro","doc":""},{"ref":"RTypes.html#derive!/3","title":"RTypes.derive!/3","type":"function","doc":""},{"ref":"RTypes.html#format_error_description/1","title":"RTypes.format_error_description/1","type":"function","doc":""},{"ref":"RTypes.html#make_predicate/1","title":"RTypes.make_predicate/1","type":"macro","doc":"Derive a predicate for the given type expression. iex&gt; require RTypes iex&gt; non_neg_integer? = RTypes.make_predicate(non_neg_integer()) iex&gt; non_neg_integer?.(10) true iex&gt; non_neg_integer?.(0) true iex&gt; non_neg_integer?.(-3) false iex&gt; non_neg_integer?.(:ok) false"},{"ref":"RTypes.html#make_predicate/3","title":"RTypes.make_predicate/3","type":"function","doc":"Return a predicate given a module name, type name, and type parameters. The predicate behaves the same way as the one produced by make_predicate/1 macro."},{"ref":"RTypes.html#make_validator/1","title":"RTypes.make_validator/1","type":"macro","doc":"Derive a validtation function for the given type expression. Usage iex&gt; require RTypes iex&gt; validate_port_number = RTypes.make_validator(:inet.port_number()) iex&gt; validate_port_number.(8080) :ok iex&gt; match?({:error, _}, validate_port_number.(70000)) true iex&gt; validate_kw_list = RTypes.make_validator(Keyword.t(pos_integer())) iex&gt; validate_kw_list.([a: 1, b: 2]) :ok Note that the macro expects its argument provided as in MyModule.my_type(arg1, arg2) The returned function either returns :ok or {:error, reason} where reason details what went wrong."},{"ref":"RTypes.html#make_validator/3","title":"RTypes.make_validator/3","type":"function","doc":"Derive a validation function given a module name, type name, and type parameters. Type parameters must be of some concrete type. Example iex&gt; validate_kw_list = RTypes.make_validator(Keyword, :t, [{:type, 0, :pos_integer, []}]) iex&gt; validate_kw_list.(key1: 4, key2: 5) :ok The function returns either :ok or {:error, error_description} where error_description details what went wrong."},{"ref":"RTypes.html#t:error_description/0","title":"RTypes.error_description/0","type":"type","doc":"t:error_decription/1 is a keyword list which details the validation error."},{"ref":"RTypes.Checker.html","title":"RTypes.Checker","type":"module","doc":""},{"ref":"RTypes.Checker.html#check/2","title":"RTypes.Checker.check/2","type":"function","doc":"Ensure that term corresponds to the given type. The function either returns :ok or {:error, reason}, where reason explains what went wrong."},{"ref":"RTypes.Extractor.html","title":"RTypes.Extractor","type":"module","doc":""},{"ref":"RTypes.Extractor.html#extract_type/3","title":"RTypes.Extractor.extract_type/3","type":"function","doc":"Recursively extract and instantiate AST representation of a type. Arguments mod - module type_name - type name type_args - arguments, if any, for the type. The arguments should be represented as AST, for example, {:type, 0, :list, []} Usage iex&gt; t = RTypes.Extractor.extract_type(:inet, :port_number, []) iex&gt; match?({:type, _, :range, [{:integer, _, 0}, {:integer, _, 65535}]}, t) true iex&gt; t = RTypes.Extractor.extract_type(Keyword, :t, [{:type, 0, :list, []}]) iex&gt; match?({:type, _, :list, [{:type, _, :tuple, [{:type, _, :atom, []}, {:type, _, :list, []}]}]}, t) true"},{"ref":"RTypes.Extractor.html#t:type/0","title":"RTypes.Extractor.type/0","type":"type","doc":""},{"ref":"RTypes.Extractor.html#t:unfolded_type/0","title":"RTypes.Extractor.unfolded_type/0","type":"type","doc":""},{"ref":"RTypes.Extractor.html#t:value/0","title":"RTypes.Extractor.value/0","type":"type","doc":""},{"ref":"RTypes.Lambda.html","title":"RTypes.Lambda","type":"module","doc":""},{"ref":"RTypes.Lambda.html#build/1","title":"RTypes.Lambda.build/1","type":"function","doc":""}]